@@ -32,3 +32,5 @@ EXPORT_CERTIFICATE = "export certificate"
 REMOVE_CERTIFICATE = "remove certificate"
 REQUEST_ACTION = "request_action"
 PUSH = "push"
+# demand register
+NEXT_PERIOD = "next_period"
