@@ -28,7 +28,6 @@ from .client_setup import ClientSetup
 from .clock import Clock
 from .data import Data
 from .disconnect_control import DisconnectControl
-from .extended_register import ExtendedRegister
 from .gprs_modem_setup import GPRSModemSetup
 from .gsm_diagnostic.ver0 import GSMDiagnostic as GSMDiagnosticVer0
 from .gsm_diagnostic.ver1 import GSMDiagnostic as GSMDiagnosticVer1
@@ -43,6 +42,8 @@ from .limiter import Limiter
 from .profile_generic.ver0 import ProfileGeneric as ProfileGenericVer0
 from .profile_generic.ver1 import ProfileGeneric as ProfileGenericVer1
 from .register import Register
+from .extended_register import ExtendedRegister
+from .demand_register.ver0 import DemandRegister as DemandRegisterVer0
 from .register_monitor import RegisterMonitor
 from .schedule import Schedule
 from .security_setup.ver0 import SecuritySetup as SecuritySetupVer0
@@ -53,12 +54,14 @@ from .special_days_table import SpecialDaysTable
 from .tcp_udp_setup import TCPUDPSetup
 from .. import ITE_exceptions as exc
 import xml.etree.ElementTree as ET
+from xml.dom import minidom
 from ..relation_to_OBIS import get_name
 from ..cosem_interface_classes import implementations as impl
 from ..cosem_interface_classes.overview import ClassID, Version, CountrySpecificIdentifiers
 from ..enums import TagsName, MechanismId
 from . import obis as o
 from .. import pdu_enums as pdu
+from ..configure import get_saved_parameters
 
 AssociationSN: TypeAlias = AssociationSNVer0
 AssociationLN: TypeAlias = AssociationLNVer0 | AssociationLNVer1 | AssociationLNVer2
@@ -66,11 +69,12 @@ ModemConfiguration: TypeAlias = PSTNModemConfiguration | ModemConfigurationVer1
 SecuritySetup: TypeAlias = SecuritySetupVer0 | SecuritySetupVer1
 PushSetup: TypeAlias = PushSetupVer0 | PushSetupVer1 | PushSetupVer2
 ProfileGeneric: TypeAlias = ProfileGenericVer1
+DemandRegister: TypeAlias = DemandRegisterVer0
 IECHDLCSetup: TypeAlias = IECHDLCSetupVer0 | IECHDLCSetupVer1
 GSMDiagnostic: TypeAlias = GSMDiagnosticVer0 | GSMDiagnosticVer1 | GSMDiagnosticVer2
-InterfaceClass: TypeAlias = Data | Register | ProfileGeneric | Clock | ScriptTable | Schedule | SpecialDaysTable | ActivityCalendar | SingleActionSchedule | AssociationLN | \
-                            IECHDLCSetup | ExtendedRegister | DisconnectControl | Limiter | ModemConfiguration | PSTNModemConfiguration | ImageTransfer | GPRSModemSetup | \
-                            GSMDiagnostic | ClientSetup | SecuritySetup | TCPUDPSetup | IPv4Setup | Arbitrator | RegisterMonitor | PushSetup | AssociationSN
+InterfaceClass: TypeAlias = Data | Register | ExtendedRegister | DemandRegister | ProfileGeneric | Clock | ScriptTable | Schedule | SpecialDaysTable | ActivityCalendar | \
+                            SingleActionSchedule | AssociationLN | IECHDLCSetup | DisconnectControl | Limiter | ModemConfiguration | PSTNModemConfiguration | ImageTransfer | \
+                            GPRSModemSetup | GSMDiagnostic | ClientSetup | SecuritySetup | TCPUDPSetup | IPv4Setup | Arbitrator | RegisterMonitor | PushSetup | AssociationSN
 
 
 class ClassMap(dict):
@@ -84,6 +88,8 @@ RegisterMap = ClassMap({
     0: Register})
 ExtendedRegisterMap = ClassMap({
     0: ExtendedRegister})
+DemandRegisterMap = ClassMap({
+    0: DemandRegisterVer0})
 ProfileGenericMap = ClassMap({
     0: ProfileGenericVer0,
     1: ProfileGenericVer1})
@@ -165,9 +171,10 @@ UnsignedDataMap = ClassMap({
     0: impl.data.Unsigned
 })
 
-CosemClassMap: TypeAlias = DataMap | RegisterMap | ExtendedRegisterMap | ProfileGenericMap | ClockMap | ScriptTableMap | ScheduleMap | SpecialDaysTableMap | AssociationLNMap | \
-                           ImageTransferMap | ActivityCalendarMap | RegisterMonitorMap | SingleActionScheduleMap | IECHDLCSetupMap | ModemConfigurationMap | TCPUDPSetupMap | \
-                           IPv4SetupMap | GPRSModemSetupMap | GSMDiagnosticMap | SecuritySetupMap | ArbitratorMap | DisconnectControlMap | LimiterMap | ClientSetupMap
+CosemClassMap: TypeAlias = DataMap | RegisterMap | ExtendedRegisterMap | DemandRegisterMap | ProfileGenericMap | ClockMap | ScriptTableMap | ScheduleMap | SpecialDaysTableMap | \
+                           AssociationLNMap | ImageTransferMap | ActivityCalendarMap | RegisterMonitorMap | SingleActionScheduleMap | IECHDLCSetupMap | ModemConfigurationMap | \
+                           TCPUDPSetupMap | IPv4SetupMap | GPRSModemSetupMap | GSMDiagnosticMap | SecuritySetupMap | ArbitratorMap | DisconnectControlMap | LimiterMap | \
+                           ClientSetupMap
 
 
 LN_C: TypeAlias = int
@@ -178,6 +185,7 @@ common_interface_class_map: dict[int, dict[[int, None], Type[InterfaceClass]]] =
     1: DataMap,
     3: RegisterMap,
     4: ExtendedRegisterMap,
+    5: DemandRegisterMap,
     7: ProfileGenericMap,
     8: ClockMap,
     9: ScriptTableMap,
@@ -330,6 +338,7 @@ __func_map_for_create: dict[FOR_C | FOR_CD | FOR_CDE | FOR_BCDE, tuple[CosemClas
     # abstract
     (0, 0, 1): DataMap,
     (0, 0, 2): DataMap,
+    (0, 0, 2, 1): ClassMap({0: impl.data.ActiveFirmwareId}),
     (0, 0, 9): DataMap,
     (0, 1, 0): ClockMap,
     (0, 1, 1): DataMap,
@@ -376,7 +385,7 @@ __func_map_for_create: dict[FOR_C | FOR_CD | FOR_CDE | FOR_BCDE, tuple[CosemClas
     #
     (0, 0, 40, 0, tuple(range(8))): (AssociationSNMap, AssociationLNMap),  # todo: now limit by 8 association, solve it
     #
-    (0, 0, 42, 0, 0): DataMap,
+    (0, 0, 42, 0, 0): ClassMap({0: impl.data.LDN}),
     (0, 0, 43, 0, tuple(range(256))): SecuritySetupMap,
     (0, 43, 1): DataMap,
     #
@@ -431,7 +440,7 @@ __func_map_for_create: dict[FOR_C | FOR_CD | FOR_CDE | FOR_BCDE, tuple[CosemClas
                                                            _UNDER_OVER_LIMIT_DURATIONS, _UNDER_OVER_LIMIT_MAGNITUDES))): (RegisterMap, ExtendedRegisterMap),
     (1, _NOT_PROCESSING_OF_MEASUREMENT_VALUES, _INSTANTANEOUS_VALUES): RegisterMap,
     (1, _NOT_PROCESSING_OF_MEASUREMENT_VALUES, _MAX_MIN_VALUES): (RegisterMap, ExtendedRegisterMap, ProfileGenericMap),
-    (1, _NOT_PROCESSING_OF_MEASUREMENT_VALUES, _CURRENT_AND_LAST_AVERAGE_VALUES): (RegisterMap, ExtendedRegisterMap),  # TODO: add DemandRegister below
+    (1, _NOT_PROCESSING_OF_MEASUREMENT_VALUES, _CURRENT_AND_LAST_AVERAGE_VALUES): (RegisterMap, DemandRegisterMap),
     (1, _NOT_PROCESSING_OF_MEASUREMENT_VALUES, 40): (DataMap, RegisterMap),
 }
 
@@ -444,6 +453,7 @@ __func_map_for_create.update({
 
 # SPODES3 Update
 __func_map_for_create.update({
+    (0, 96, 1, 6): ClassMap({0: impl.data.SPODES3SPODESVersion}),
     (0, 96, 2, (1, 2, 3, 5, 6, 7, 11, 12)): ClassMap({0: impl.data.AnyDateTime}),
     (0, 96, 3, 20): ClassMap({0: impl.arbitrator.SPODES3Arbitrator}),
     (0, 96, 5, 132): ClassMap({0: impl.data.Unsigned}),  # TODO: make according with СПОДЭС3 13.9. Контроль чередования фаз
@@ -524,24 +534,13 @@ def get_type(class_id: ut.CosemClassId,
                                ver=version)
 
 
-#  Todo: rewrite this!!!!
-def __get_manufacture_128(class_id: ut.CosemClassId,
-                          version: cdt.Unsigned | None,
-                          ln: cst.LogicalName) -> Type[InterfaceClass]:
-    match class_id, int(version), ln.b, ln.c, ln.d, ln.e:
-        case ClassID.REGISTER, 0,                                       128, 0, c, 0, 0 if c <= 19:
-            return Register
-        case _:
-            raise exc.NoObject(F'DLMS Object: {class_id=} {version=} {ln=} not searched in relation library')
-
-
 class Collection:
-    dlms_ver: int
-    manufacturer: bytes | None
-    country: CountrySpecificIdentifiers
-    country_ver: AppVersion | None
-    server_type: cdt.CommonDataType | None
-    server_ver: AppVersion | None
+    __dlms_ver: int
+    __manufacturer: bytes | None
+    __country: CountrySpecificIdentifiers
+    __country_ver: AppVersion | None
+    __server_type: cdt.CommonDataType | None
+    __server_ver: dict[int, AppVersion]
     __container: deque[InterfaceClass]
     __const_objs: list[ic.COSEMInterfaceClasses]
     __spec: str
@@ -553,18 +552,101 @@ class Collection:
         self.__const_objs = list()
         """ container for major(constant) DLMS objects LN. They don't deletable """
 
-        self.dlms_ver = 6
-        self.manufacturer = None
-        self.country = country
-        self.country_ver = None
+        self.__dlms_ver = 6
+        self.__manufacturer = None
+        self.__country = country
+        self.__country_ver = None
         """country version specification"""
-        self.server_type = None
-        self.server_ver = None
+        self.__server_type = None
+        self.__server_ver = dict()
+        """key: instance of 0.b.2.0.1.255, value AppVersion"""
         self.__spec = "DLMS_6"
 
+    @property
+    def dlms_ver(self):
+        return self.__dlms_ver
+
+    def set_dlms_ver(self, value: int):
+        if not self.__dlms_ver:
+            self.__dlms_ver = value
+        else:
+            if value != self.__dlms_ver:
+                raise ValueError(F"got dlms_version: {value}, expected {self.__dlms_ver}")
+            else:
+                """success validation"""
+
+    @property
+    def manufacturer(self):
+        return self.__manufacturer
+
+    def set_manufacturer(self, value: bytes):
+        if not self.__manufacturer:
+            self.__manufacturer = value
+        else:
+            if value != self.__manufacturer:
+                raise ValueError(F"got manufacturer: {value}, expected {self.__manufacturer}")
+            else:
+                """success validation"""
+
+    @property
+    def country(self):
+        return self.__country
+
+    def set_country(self, value: CountrySpecificIdentifiers):
+        if not self.__country:
+            self.__country = value
+        else:
+            if value != self.__country:
+                raise ValueError(F"got country: {value}, expected {self.__country}")
+            else:
+                """success validation"""
+
+    @property
+    def country_ver(self):
+        return self.__country_ver
+
+    def set_country_ver(self, value: AppVersion):
+        """country version specification"""
+        if not self.__country_ver:
+            self.__country_ver = value
+        else:
+            if value != self.__country_ver:
+                raise ValueError(F"got country version: {value}, expected {self.__country_ver}")
+            else:
+                """success validation"""
+
+    @property
+    def server_type(self):
+        return self.__server_type
+
+    def set_server_type(self, value: cdt.CommonDataTypes):
+        if not self.__server_type:
+            self.__server_type = value
+        else:
+            if value != self.__server_type:
+                raise ValueError(F"got server type: {value}, expected {self.__server_type}")
+            else:
+                """success validation"""
+
+    @property
+    def server_ver(self):
+        return self.__server_ver
+
+    def clear_server_ver(self):
+        self.__server_ver.clear()
+
+    def set_server_ver(self, instance: int, value: AppVersion):
+        if self.__server_ver.get(instance) is None:
+            self.__server_ver[instance] = value
+        else:
+            if value != self.__server_ver[instance]:
+                raise ValueError(F"got server version[{instance}]: {value}, expected {self.__server_ver[instance]}. Execute search type")
+            else:
+                """success validation"""
+
     def __str__(self):
-        return F"[{len(self.__container)}] DLMS version: {self.dlms_ver}, country: {self.country.name}, country specific version: {self.country_ver}, " \
-               F"manufacturer: {self.manufacturer}, server type: {repr(self.server_type)}, server version: {self.server_ver}, uses specification: {self.__spec}"
+        return F"[{len(self.__container)}] DLMS version: {self.__dlms_ver}, country: {self.__country.name}, country specific version: {self.__country_ver}, " \
+               F"manufacturer: {self.__manufacturer}, server type: {repr(self.__server_type)}, server version: {self.__server_ver}, uses specification: {self.__spec}"
 
     def __iter__(self) -> Iterator[ic.COSEMInterfaceClasses]:
         return iter(self.__container)
@@ -591,12 +673,20 @@ class Collection:
         if use is None and objects.tag != TagsName.DEVICE_ROOT.value:
             raise ValueError(F"ERROR: Root tag got {objects.tag}, expected {TagsName.DEVICE_ROOT.value}")
         root_version: AppVersion = AppVersion.from_str(objects.attrib.get('version', '1.0.0'))
-        self.dlms_ver = int(objects.findtext("dlms_ver", default="6"))
-        self.country = CountrySpecificIdentifiers(int(objects.findtext("country", default="7")))
-        self.country_ver = AppVersion.from_str(objects.findtext("country_ver", default="3.0"))
-        self.manufacturer = objects.findtext("manufacturer", default="").encode("utf-8")
-        self.server_type, _ = cdt.get_instance_and_pdu_from_value(bytes.fromhex(objects.findtext("server_type", default="00")))
-        self.server_ver = AppVersion.from_str(objects.findtext("server_ver", default="0.0.1"))
+        if (dlms_ver := objects.findtext("dlms_ver")) is not None:
+            self.set_dlms_ver(int(dlms_ver))
+        if (country := objects.findtext("country")) is not None:
+            self.set_country(CountrySpecificIdentifiers(int(country)))
+        if (country_ver := objects.findtext("country_ver")) is not None:
+            self.set_country_ver(AppVersion.from_str(country_ver))
+        if (manufacturer := objects.findtext("manufacturer")) is not None:
+            self.set_manufacturer(manufacturer.encode("utf-8"))
+        if (server_type := objects.findtext("server_type")) is not None:
+            tmp, _ = cdt.get_instance_and_pdu_from_value(bytes.fromhex(server_type))
+            self.set_server_type(tmp)
+        for server_ver in objects.findall("server_ver"):
+            self.set_server_ver(instance=int(server_ver.attrib.get("instance", "0")),
+                                value=AppVersion.from_str(server_ver.text))
         self.set_spec()
         logger.info(F'Версия: {root_version}, file: {filename.split("/")[-1]}')
         match root_version:
@@ -674,6 +764,100 @@ class Collection:
             case _ as error:
                 raise exc.VersionError(error, additional='Xml')
 
+    def __get_base_xml_element(self, root_tag: str = TagsName.DEVICE_ROOT.value) -> ET.Element:
+        objects = ET.Element(root_tag, attrib={'version': '3.1.0'})
+        ET.SubElement(objects, 'dlms_ver').text = str(self.dlms_ver)
+        ET.SubElement(objects, 'country').text = str(self.country.value)
+        if self.country_ver:
+            ET.SubElement(objects, 'country_ver').text = str(self.country_ver)
+        if self.manufacturer is not None:
+            ET.SubElement(objects, 'manufacturer').text = self.manufacturer.decode("utf-8")
+        if self.server_type is not None:
+            ET.SubElement(objects, 'server_type').text = self.server_type.encoding.hex()
+        for ver in self.server_ver:
+            server_ver_node = ET.SubElement(objects, 'server_ver', attrib={"instance": str(ver)})
+            server_ver_node.text = str(self.server_ver[ver])
+        return objects
+
+    def to_xml(self, file_name: str,
+               root_tag: str = TagsName.DEVICE_ROOT.value,
+               with_comment: bool = False):
+        """Save attributes of client. For types only STATIC save """
+        classes: set[ut.CosemClassId] = set()
+        objects = self.__get_base_xml_element(root_tag)
+        for obj in self.values():
+            object_node = ET.SubElement(objects, 'object', attrib={'name': F'{get_name(obj.logical_name)}', 'ln': str(obj.logical_name)})
+            ET.SubElement(object_node, 'class_id').text = str(obj.CLASS_ID)
+            if obj.CLASS_ID not in classes:
+                ET.SubElement(object_node, 'version').text = str(obj.VERSION)
+            classes.add(obj.CLASS_ID)
+            if root_tag == TagsName.DEVICE_ROOT.value and obj is self.current_association:
+                ET.SubElement(object_node, 'attribute', attrib={'index': '3'}).text = obj.associated_partners_id.encoding.hex()
+                continue
+            for index, attr in obj.get_index_with_attributes():
+                if index == 1:  # don't keep ln
+                    continue
+                el = obj.get_attr_element(index)
+                match attr, el.DATA_TYPE:
+                    case None, ut.CHOICE:
+                        logger.warning(F'PASS choice {obj} {index}')
+                    case None, cdt.CommonDataType():
+                        if with_comment:
+                            object_node.append(ET.Comment(F'{el.NAME}. Type: {el.DATA_TYPE}'))
+                        ET.SubElement(object_node, 'attribute', attrib={'index': str(index)}).text = str(el.DATA_TYPE.TAG[0])
+                    case cdt.CommonDataType(), _:
+                        record_time = obj.get_record_time(index)
+                        el_attrib: dict = {'index': str(index)}
+                        if record_time is not None:
+                            el_attrib.update({'rec_time': record_time.encoding.hex()})
+                        if with_comment:
+                            object_node.append(ET.Comment(F'{el.NAME}: {attr}'))
+                        ET.SubElement(object_node, 'attribute', attrib=el_attrib).text = attr.encoding.hex()
+                    case _:
+                        logger.warning('PASS')
+
+        # TODO: '<!DOCTYPE ITE_util_tree SYSTEM "setting.dtd"> or xsd
+        xml_string = ET.tostring(objects, encoding='cp1251', method='xml')
+        dom_xml = minidom.parseString(xml_string)
+        str_ = dom_xml.toprettyxml(indent="  ", encoding='cp1251')
+        with open(file_name, "wb") as f:
+            f.write(str_)
+
+    def save_type(self, file_name: str, root_tag: str = TagsName.DEVICE_ROOT.value):
+        """ For concrete device save all attributes. For types only STATIC save """
+        objects = self.__get_base_xml_element(root_tag)
+        classes: set[ut.CosemClassId] = set()
+        for obj in self.values():
+            object_node = ET.SubElement(objects, 'object', attrib={'name': F'{get_name(obj.logical_name)}', 'ln': str(obj.logical_name)})
+            ET.SubElement(object_node, 'class_id').text = str(obj.CLASS_ID)
+            if obj.CLASS_ID not in classes:
+                ET.SubElement(object_node, 'version').text = str(obj.VERSION)
+            classes.add(obj.CLASS_ID)
+            for index, method in get_saved_parameters(obj).items():
+                attr = obj.get_attr(index)
+                match method, attr:
+                    case 1, None if isinstance(obj.get_attr_element(index).DATA_TYPE, ut.CHOICE):
+                        logger.warning(F'For {obj} {attr} not selected type from: {obj.get_attr_element(index).DATA_TYPE}')
+                    case 1, None:
+                        logger.warning(F'REMOVE {obj} {attr} for saving in type')
+                    case 1, _:
+                        object_node.append(ET.Comment(F'{obj.get_attr_element(index).NAME}. Type: {attr.NAME}'))
+                        ET.SubElement(object_node, 'attribute', attrib={'index': str(index)}).text = str(attr.TAG[0])
+                    case 0, cdt.CommonDataType():
+                        object_node.append(ET.Comment(F'{obj.get_attr_element(index).NAME}: {attr}'))
+                        ET.SubElement(object_node, 'attribute', attrib={'index': str(index)}).text = attr.encoding.hex()
+                    case 0, None:
+                        logger.warning(F'For {obj} attr: {index} value not set')
+                    case _ as unknown:
+                        logger.warning(F'Unknown pattern for select keep attribute: {unknown}')
+
+        # TODO: '<!DOCTYPE ITE_util_tree SYSTEM "setting.dtd"> or xsd
+        xml_string = ET.tostring(objects, encoding='cp1251', method='xml')
+        dom_xml = minidom.parseString(xml_string)
+        str_ = dom_xml.toprettyxml(indent="  ", encoding='cp1251')
+        with open(file_name, "wb") as f:
+            f.write(str_)
+
     def add_major(self, obj: InterfaceClass):
         self.__const_objs.append(obj)
 
@@ -686,7 +870,7 @@ class Collection:
             if self.country_ver == AppVersion(3, 0):
                 self.__spec = "SPODES_3"
             if self.manufacturer == b"KPZ":
-                if self.server_ver < AppVersion(1, 4, 0):
+                if self.server_ver.get(0) < AppVersion(1, 4, 0):
                     self.__spec = "KPZ1"
                 else:
                     self.__spec = "KPZ"
@@ -700,7 +884,10 @@ class Collection:
         """ TODO: naming"""
         if version is None:
             version = self.set_version(class_id, version)
-        return get_type(class_id, version, ln, func_maps[self.__spec])(ln)
+        try:
+            return get_type(class_id, version, ln, func_maps[self.__spec])(ln)
+        except ValueError as e:
+            raise ValueError(F"error getting DLMS object instance with {class_id=} {version=} {ln=}: {e}")
         # ret: Type[InterfaceClass] = _func_map_A.get(ln.a, None)
         # if ret:
         #     return ret(class_id, version, ln)(ln)
@@ -893,7 +1080,7 @@ class Collection:
             return obj
 
     @cached_property
-    def LDN(self) -> Data:
+    def LDN(self) -> impl.data.LDN:
         return self.__get_object(o.LDN)
 
     @cached_property
